/* DataViewTypes.c generated by valac 0.56.19, the Vala compiler
 * generated from DataViewTypes.vala, do not modify */

/*
* Copyright (c) 2011-2013 Yorba Foundation
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*/

#include "io.elementary.photos.h"
#include <glib-object.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	THUMBNAIL_VIEW_0_PROPERTY,
	THUMBNAIL_VIEW_NUM_PROPERTIES
};
static GParamSpec* thumbnail_view_properties[THUMBNAIL_VIEW_NUM_PROPERTIES];
enum  {
	THUMBNAIL_VIEW_THUMBNAIL_ALTERED_SIGNAL,
	THUMBNAIL_VIEW_NUM_SIGNALS
};
static guint thumbnail_view_signals[THUMBNAIL_VIEW_NUM_SIGNALS] = {0};
enum  {
	PHOTO_VIEW_0_PROPERTY,
	PHOTO_VIEW_NUM_PROPERTIES
};
static GParamSpec* photo_view_properties[PHOTO_VIEW_NUM_PROPERTIES];
enum  {
	VIDEO_VIEW_0_PROPERTY,
	VIDEO_VIEW_NUM_PROPERTIES
};
static GParamSpec* video_view_properties[VIDEO_VIEW_NUM_PROPERTIES];
enum  {
	EVENT_VIEW_0_PROPERTY,
	EVENT_VIEW_NUM_PROPERTIES
};
static GParamSpec* event_view_properties[EVENT_VIEW_NUM_PROPERTIES];

static gpointer thumbnail_view_parent_class = NULL;
static gpointer photo_view_parent_class = NULL;
static gpointer video_view_parent_class = NULL;
static gpointer event_view_parent_class = NULL;

static void thumbnail_view_real_notify_thumbnail_altered (ThumbnailView* self);
static void thumbnail_view_real_thumbnail_altered (ThumbnailView* self);
static GType thumbnail_view_get_type_once (void);
static GType photo_view_get_type_once (void);
static GType video_view_get_type_once (void);
static GType event_view_get_type_once (void);

ThumbnailView*
thumbnail_view_construct (GType object_type,
                          ThumbnailSource* source)
{
	ThumbnailView * self = NULL;
	g_return_val_if_fail (source != NULL, NULL);
	self = (ThumbnailView*) g_object_new (object_type, "source", source, NULL);
	return self;
}

ThumbnailView*
thumbnail_view_new (ThumbnailSource* source)
{
	return thumbnail_view_construct (TYPE_THUMBNAIL_VIEW, source);
}

static void
thumbnail_view_real_notify_thumbnail_altered (ThumbnailView* self)
{
	g_signal_emit (self, thumbnail_view_signals[THUMBNAIL_VIEW_THUMBNAIL_ALTERED_SIGNAL], 0);
}

void
thumbnail_view_notify_thumbnail_altered (ThumbnailView* self)
{
	ThumbnailViewClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = THUMBNAIL_VIEW_GET_CLASS (self);
	if (_klass_->notify_thumbnail_altered) {
		_klass_->notify_thumbnail_altered (self);
	}
}

static void
thumbnail_view_real_thumbnail_altered (ThumbnailView* self)
{
}

static void
thumbnail_view_class_init (ThumbnailViewClass * klass,
                           gpointer klass_data)
{
	thumbnail_view_parent_class = g_type_class_peek_parent (klass);
	((ThumbnailViewClass *) klass)->notify_thumbnail_altered = (void (*) (ThumbnailView*)) thumbnail_view_real_notify_thumbnail_altered;
	((ThumbnailViewClass *) klass)->thumbnail_altered = thumbnail_view_real_thumbnail_altered;
	thumbnail_view_signals[THUMBNAIL_VIEW_THUMBNAIL_ALTERED_SIGNAL] = g_signal_new ("thumbnail-altered", TYPE_THUMBNAIL_VIEW, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (ThumbnailViewClass, thumbnail_altered), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}

static void
thumbnail_view_instance_init (ThumbnailView * self,
                              gpointer klass)
{
}

static GType
thumbnail_view_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ThumbnailViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) thumbnail_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ThumbnailView), 0, (GInstanceInitFunc) thumbnail_view_instance_init, NULL };
	GType thumbnail_view_type_id;
	thumbnail_view_type_id = g_type_register_static (TYPE_DATA_VIEW, "ThumbnailView", &g_define_type_info, 0);
	return thumbnail_view_type_id;
}

GType
thumbnail_view_get_type (void)
{
	static volatile gsize thumbnail_view_type_id__once = 0;
	if (g_once_init_enter (&thumbnail_view_type_id__once)) {
		GType thumbnail_view_type_id;
		thumbnail_view_type_id = thumbnail_view_get_type_once ();
		g_once_init_leave (&thumbnail_view_type_id__once, thumbnail_view_type_id);
	}
	return thumbnail_view_type_id__once;
}

PhotoView*
photo_view_construct (GType object_type,
                      PhotoSource* source)
{
	PhotoView * self = NULL;
	g_return_val_if_fail (source != NULL, NULL);
	self = (PhotoView*) g_object_new (object_type, "source", source, NULL);
	return self;
}

PhotoView*
photo_view_new (PhotoSource* source)
{
	return photo_view_construct (TYPE_PHOTO_VIEW, source);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

PhotoSource*
photo_view_get_photo_source (PhotoView* self)
{
	DataSource* _tmp0_;
	DataSource* _tmp1_;
	PhotoSource* _tmp2_;
	PhotoSource* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = data_view_get_source ((DataView*) self);
	_tmp1_ = _tmp0_;
	_tmp2_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, TYPE_PHOTO_SOURCE, PhotoSource));
	result = _tmp2_;
	return result;
}

static void
photo_view_class_init (PhotoViewClass * klass,
                       gpointer klass_data)
{
	photo_view_parent_class = g_type_class_peek_parent (klass);
}

static void
photo_view_instance_init (PhotoView * self,
                          gpointer klass)
{
}

static GType
photo_view_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (PhotoViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) photo_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PhotoView), 0, (GInstanceInitFunc) photo_view_instance_init, NULL };
	GType photo_view_type_id;
	photo_view_type_id = g_type_register_static (TYPE_THUMBNAIL_VIEW, "PhotoView", &g_define_type_info, 0);
	return photo_view_type_id;
}

GType
photo_view_get_type (void)
{
	static volatile gsize photo_view_type_id__once = 0;
	if (g_once_init_enter (&photo_view_type_id__once)) {
		GType photo_view_type_id;
		photo_view_type_id = photo_view_get_type_once ();
		g_once_init_leave (&photo_view_type_id__once, photo_view_type_id);
	}
	return photo_view_type_id__once;
}

VideoView*
video_view_construct (GType object_type,
                      VideoSource* source)
{
	VideoView * self = NULL;
	g_return_val_if_fail (source != NULL, NULL);
	self = (VideoView*) g_object_new (object_type, "source", source, NULL);
	return self;
}

VideoView*
video_view_new (VideoSource* source)
{
	return video_view_construct (TYPE_VIDEO_VIEW, source);
}

VideoSource*
video_view_get_video_source (VideoView* self)
{
	DataSource* _tmp0_;
	DataSource* _tmp1_;
	VideoSource* _tmp2_;
	VideoSource* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = data_view_get_source ((DataView*) self);
	_tmp1_ = _tmp0_;
	_tmp2_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, TYPE_VIDEO_SOURCE, VideoSource));
	result = _tmp2_;
	return result;
}

static void
video_view_class_init (VideoViewClass * klass,
                       gpointer klass_data)
{
	video_view_parent_class = g_type_class_peek_parent (klass);
}

static void
video_view_instance_init (VideoView * self,
                          gpointer klass)
{
}

static GType
video_view_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (VideoViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) video_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VideoView), 0, (GInstanceInitFunc) video_view_instance_init, NULL };
	GType video_view_type_id;
	video_view_type_id = g_type_register_static (TYPE_THUMBNAIL_VIEW, "VideoView", &g_define_type_info, 0);
	return video_view_type_id;
}

GType
video_view_get_type (void)
{
	static volatile gsize video_view_type_id__once = 0;
	if (g_once_init_enter (&video_view_type_id__once)) {
		GType video_view_type_id;
		video_view_type_id = video_view_get_type_once ();
		g_once_init_leave (&video_view_type_id__once, video_view_type_id);
	}
	return video_view_type_id__once;
}

EventView*
event_view_construct (GType object_type,
                      EventSource* source)
{
	EventView * self = NULL;
	g_return_val_if_fail (source != NULL, NULL);
	self = (EventView*) g_object_new (object_type, "source", source, NULL);
	return self;
}

EventView*
event_view_new (EventSource* source)
{
	return event_view_construct (TYPE_EVENT_VIEW, source);
}

EventSource*
event_view_get_event_source (EventView* self)
{
	DataSource* _tmp0_;
	DataSource* _tmp1_;
	EventSource* _tmp2_;
	EventSource* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = data_view_get_source ((DataView*) self);
	_tmp1_ = _tmp0_;
	_tmp2_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, TYPE_EVENT_SOURCE, EventSource));
	result = _tmp2_;
	return result;
}

static void
event_view_class_init (EventViewClass * klass,
                       gpointer klass_data)
{
	event_view_parent_class = g_type_class_peek_parent (klass);
}

static void
event_view_instance_init (EventView * self,
                          gpointer klass)
{
}

static GType
event_view_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (EventViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) event_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (EventView), 0, (GInstanceInitFunc) event_view_instance_init, NULL };
	GType event_view_type_id;
	event_view_type_id = g_type_register_static (TYPE_THUMBNAIL_VIEW, "EventView", &g_define_type_info, 0);
	return event_view_type_id;
}

GType
event_view_get_type (void)
{
	static volatile gsize event_view_type_id__once = 0;
	if (g_once_init_enter (&event_view_type_id__once)) {
		GType event_view_type_id;
		event_view_type_id = event_view_get_type_once ();
		g_once_init_leave (&event_view_type_id__once, event_view_type_id);
	}
	return event_view_type_id__once;
}

