/* DataView.c generated by valac 0.56.19, the Vala compiler
 * generated from DataView.vala, do not modify */

/*
* Copyright (c) 2011-2013 Yorba Foundation
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*/

#include "io.elementary.photos.h"
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	DATA_VIEW_0_PROPERTY,
	DATA_VIEW_SOURCE_PROPERTY,
	DATA_VIEW_NUM_PROPERTIES
};
static GParamSpec* data_view_properties[DATA_VIEW_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _data_collection_unref0(var) ((var == NULL) ? NULL : (var = (data_collection_unref (var), NULL)))
enum  {
	DATA_VIEW_STATE_CHANGED_SIGNAL,
	DATA_VIEW_VISIBILITY_CHANGED_SIGNAL,
	DATA_VIEW_VIEW_ALTERED_SIGNAL,
	DATA_VIEW_GEOMETRY_ALTERED_SIGNAL,
	DATA_VIEW_UNSUBSCRIBED_SIGNAL,
	DATA_VIEW_NUM_SIGNALS
};
static guint data_view_signals[DATA_VIEW_NUM_SIGNALS] = {0};

struct _DataViewPrivate {
	DataSource* _source;
	gboolean selected;
	gboolean visible;
};

static gint DataView_private_offset;
static gpointer data_view_parent_class = NULL;

static gchar* data_view_real_get_name (DataObject* base);
static gchar* data_view_real_to_string (DataObject* base);
static void data_view_real_notify_view_altered (DataView* self);
static void data_view_real_notify_geometry_altered (DataView* self);
static void data_view_real_notify_unsubscribed (DataView* self,
                                         DataSource* source);
static void data_view_real_state_changed (DataView* self,
                                   gboolean selected);
static void data_view_real_visibility_changed (DataView* self,
                                        gboolean visible);
static void data_view_real_view_altered (DataView* self);
static void data_view_real_geometry_altered (DataView* self);
static void data_view_real_unsubscribed (DataView* self,
                                  DataSource* source);
static GObject * data_view_constructor (GType type,
                                 guint n_construct_properties,
                                 GObjectConstructParam * construct_properties);
static void data_view_finalize (GObject * obj);
static GType data_view_get_type_once (void);
static void _vala_data_view_get_property (GObject * object,
                                   guint property_id,
                                   GValue * value,
                                   GParamSpec * pspec);
static void _vala_data_view_set_property (GObject * object,
                                   guint property_id,
                                   const GValue * value,
                                   GParamSpec * pspec);

static inline gpointer
data_view_get_instance_private (DataView* self)
{
	return G_STRUCT_MEMBER_P (self, DataView_private_offset);
}

DataView*
data_view_construct (GType object_type,
                     DataSource* source)
{
	DataView * self = NULL;
	g_return_val_if_fail (source != NULL, NULL);
	self = (DataView*) g_object_new (object_type, "source", source, NULL);
	return self;
}

DataView*
data_view_new (DataSource* source)
{
	return data_view_construct (TYPE_DATA_VIEW, source);
}

static gchar*
data_view_real_get_name (DataObject* base)
{
	DataView * self;
	DataSource* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* result;
	self = (DataView*) base;
	_tmp0_ = self->priv->_source;
	_tmp1_ = data_object_get_name ((DataObject*) _tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = g_strdup_printf ("View of %s", _tmp2_);
	_tmp4_ = _tmp3_;
	_g_free0 (_tmp2_);
	result = _tmp4_;
	return result;
}

static gchar*
data_view_real_to_string (DataObject* base)
{
	DataView * self;
	gchar* _tmp0_;
	gchar* _tmp1_;
	DataSource* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	gchar* result;
	self = (DataView*) base;
	_tmp0_ = data_object_get_name ((DataObject*) self);
	_tmp1_ = _tmp0_;
	_tmp2_ = self->priv->_source;
	_tmp3_ = data_object_to_string ((DataObject*) _tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = g_strdup_printf ("DataView %s [DataSource %s]", _tmp1_, _tmp4_);
	_tmp6_ = _tmp5_;
	_g_free0 (_tmp4_);
	_g_free0 (_tmp1_);
	result = _tmp6_;
	return result;
}

gboolean
data_view_is_selected (DataView* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->selected;
	return result;
}

void
data_view_internal_set_selected (DataView* self,
                                 gboolean selected)
{
	g_return_if_fail (self != NULL);
	if (self->priv->selected == selected) {
		return;
	}
	self->priv->selected = selected;
	g_signal_emit (self, data_view_signals[DATA_VIEW_STATE_CHANGED_SIGNAL], 0, selected);
}

gboolean
data_view_internal_toggle (DataView* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	self->priv->selected = !self->priv->selected;
	g_signal_emit (self, data_view_signals[DATA_VIEW_STATE_CHANGED_SIGNAL], 0, self->priv->selected);
	result = self->priv->selected;
	return result;
}

gboolean
data_view_is_visible (DataView* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->visible;
	return result;
}

void
data_view_internal_set_visible (DataView* self,
                                gboolean visible)
{
	g_return_if_fail (self != NULL);
	if (self->priv->visible == visible) {
		return;
	}
	self->priv->visible = visible;
	g_signal_emit (self, data_view_signals[DATA_VIEW_VISIBILITY_CHANGED_SIGNAL], 0, visible);
}

static void
data_view_real_notify_view_altered (DataView* self)
{
	ViewCollection* vc = NULL;
	DataCollection* _tmp0_;
	ViewCollection* _tmp1_;
	ViewCollection* _tmp2_;
	if (!self->priv->visible) {
		return;
	}
	_tmp0_ = data_object_get_membership ((DataObject*) self);
	_tmp1_ = IS_VIEW_COLLECTION (_tmp0_) ? ((ViewCollection*) _tmp0_) : NULL;
	if (_tmp1_ == NULL) {
		_data_collection_unref0 (_tmp0_);
	}
	vc = _tmp1_;
	_tmp2_ = vc;
	if (_tmp2_ != NULL) {
		ViewCollection* _tmp3_;
		ViewCollection* _tmp4_;
		_tmp3_ = vc;
		if (!data_collection_are_notifications_frozen ((DataCollection*) _tmp3_)) {
			g_signal_emit (self, data_view_signals[DATA_VIEW_VIEW_ALTERED_SIGNAL], 0);
		}
		_tmp4_ = vc;
		view_collection_internal_notify_view_altered (_tmp4_, self);
	} else {
		g_signal_emit (self, data_view_signals[DATA_VIEW_VIEW_ALTERED_SIGNAL], 0);
	}
	_data_collection_unref0 (vc);
}

void
data_view_notify_view_altered (DataView* self)
{
	DataViewClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = DATA_VIEW_GET_CLASS (self);
	if (_klass_->notify_view_altered) {
		_klass_->notify_view_altered (self);
	}
}

static void
data_view_real_notify_geometry_altered (DataView* self)
{
	ViewCollection* vc = NULL;
	DataCollection* _tmp0_;
	ViewCollection* _tmp1_;
	ViewCollection* _tmp2_;
	if (!self->priv->visible) {
		return;
	}
	_tmp0_ = data_object_get_membership ((DataObject*) self);
	_tmp1_ = IS_VIEW_COLLECTION (_tmp0_) ? ((ViewCollection*) _tmp0_) : NULL;
	if (_tmp1_ == NULL) {
		_data_collection_unref0 (_tmp0_);
	}
	vc = _tmp1_;
	_tmp2_ = vc;
	if (_tmp2_ != NULL) {
		ViewCollection* _tmp3_;
		ViewCollection* _tmp4_;
		_tmp3_ = vc;
		if (!data_collection_are_notifications_frozen ((DataCollection*) _tmp3_)) {
			g_signal_emit (self, data_view_signals[DATA_VIEW_GEOMETRY_ALTERED_SIGNAL], 0);
		}
		_tmp4_ = vc;
		view_collection_internal_notify_geometry_altered (_tmp4_, self);
	} else {
		g_signal_emit (self, data_view_signals[DATA_VIEW_GEOMETRY_ALTERED_SIGNAL], 0);
	}
	_data_collection_unref0 (vc);
}

void
data_view_notify_geometry_altered (DataView* self)
{
	DataViewClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = DATA_VIEW_GET_CLASS (self);
	if (_klass_->notify_geometry_altered) {
		_klass_->notify_geometry_altered (self);
	}
}

static void
data_view_real_notify_unsubscribed (DataView* self,
                                    DataSource* source)
{
	g_return_if_fail (source != NULL);
	g_signal_emit (self, data_view_signals[DATA_VIEW_UNSUBSCRIBED_SIGNAL], 0, source);
}

void
data_view_notify_unsubscribed (DataView* self,
                               DataSource* source)
{
	DataViewClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = DATA_VIEW_GET_CLASS (self);
	if (_klass_->notify_unsubscribed) {
		_klass_->notify_unsubscribed (self, source);
	}
}

DataSource*
data_view_get_source (DataView* self)
{
	DataSource* result;
	DataSource* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_source;
	result = _tmp0_;
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static void
data_view_set_source (DataView* self,
                      DataSource* value)
{
	DataSource* old_value;
	g_return_if_fail (self != NULL);
	old_value = data_view_get_source (self);
	if (old_value != value) {
		DataSource* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_source);
		self->priv->_source = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, data_view_properties[DATA_VIEW_SOURCE_PROPERTY]);
	}
}

static void
data_view_real_state_changed (DataView* self,
                              gboolean selected)
{
}

static void
data_view_real_visibility_changed (DataView* self,
                                   gboolean visible)
{
}

static void
data_view_real_view_altered (DataView* self)
{
}

static void
data_view_real_geometry_altered (DataView* self)
{
}

static void
data_view_real_unsubscribed (DataView* self,
                             DataSource* source)
{
	g_return_if_fail (source != NULL);
}

static GObject *
data_view_constructor (GType type,
                       guint n_construct_properties,
                       GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	DataView * self;
	DataSource* _tmp0_;
	parent_class = G_OBJECT_CLASS (data_view_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_DATA_VIEW, DataView);
	_tmp0_ = self->priv->_source;
	data_source_internal_subscribe (_tmp0_, self);
	return obj;
}

static void
data_view_class_init (DataViewClass * klass,
                      gpointer klass_data)
{
	data_view_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &DataView_private_offset);
	((DataObjectClass *) klass)->get_name = (gchar* (*) (DataObject*)) data_view_real_get_name;
	((DataObjectClass *) klass)->to_string = (gchar* (*) (DataObject*)) data_view_real_to_string;
	((DataViewClass *) klass)->notify_view_altered = (void (*) (DataView*)) data_view_real_notify_view_altered;
	((DataViewClass *) klass)->notify_geometry_altered = (void (*) (DataView*)) data_view_real_notify_geometry_altered;
	((DataViewClass *) klass)->notify_unsubscribed = (void (*) (DataView*, DataSource*)) data_view_real_notify_unsubscribed;
	((DataViewClass *) klass)->state_changed = data_view_real_state_changed;
	((DataViewClass *) klass)->visibility_changed = data_view_real_visibility_changed;
	((DataViewClass *) klass)->view_altered = data_view_real_view_altered;
	((DataViewClass *) klass)->geometry_altered = data_view_real_geometry_altered;
	((DataViewClass *) klass)->unsubscribed = data_view_real_unsubscribed;
	G_OBJECT_CLASS (klass)->get_property = _vala_data_view_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_data_view_set_property;
	G_OBJECT_CLASS (klass)->constructor = data_view_constructor;
	G_OBJECT_CLASS (klass)->finalize = data_view_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DATA_VIEW_SOURCE_PROPERTY, data_view_properties[DATA_VIEW_SOURCE_PROPERTY] = g_param_spec_object ("source", "source", "source", TYPE_DATA_SOURCE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	data_view_signals[DATA_VIEW_STATE_CHANGED_SIGNAL] = g_signal_new ("state-changed", TYPE_DATA_VIEW, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (DataViewClass, state_changed), NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN, G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
	data_view_signals[DATA_VIEW_VISIBILITY_CHANGED_SIGNAL] = g_signal_new ("visibility-changed", TYPE_DATA_VIEW, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (DataViewClass, visibility_changed), NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN, G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
	data_view_signals[DATA_VIEW_VIEW_ALTERED_SIGNAL] = g_signal_new ("view-altered", TYPE_DATA_VIEW, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (DataViewClass, view_altered), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	data_view_signals[DATA_VIEW_GEOMETRY_ALTERED_SIGNAL] = g_signal_new ("geometry-altered", TYPE_DATA_VIEW, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (DataViewClass, geometry_altered), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	data_view_signals[DATA_VIEW_UNSUBSCRIBED_SIGNAL] = g_signal_new ("unsubscribed", TYPE_DATA_VIEW, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (DataViewClass, unsubscribed), NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, TYPE_DATA_SOURCE);
}

static void
data_view_instance_init (DataView * self,
                         gpointer klass)
{
	self->priv = data_view_get_instance_private (self);
	self->priv->selected = FALSE;
	self->priv->visible = TRUE;
}

static void
data_view_finalize (GObject * obj)
{
	DataView * self;
	DataSource* _tmp0_;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_DATA_VIEW, DataView);
	_tmp0_ = self->priv->_source;
	data_source_internal_unsubscribe (_tmp0_, self);
	_g_object_unref0 (self->priv->_source);
	G_OBJECT_CLASS (data_view_parent_class)->finalize (obj);
}

static GType
data_view_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (DataViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) data_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DataView), 0, (GInstanceInitFunc) data_view_instance_init, NULL };
	GType data_view_type_id;
	data_view_type_id = g_type_register_static (TYPE_DATA_OBJECT, "DataView", &g_define_type_info, 0);
	DataView_private_offset = g_type_add_instance_private (data_view_type_id, sizeof (DataViewPrivate));
	return data_view_type_id;
}

GType
data_view_get_type (void)
{
	static volatile gsize data_view_type_id__once = 0;
	if (g_once_init_enter (&data_view_type_id__once)) {
		GType data_view_type_id;
		data_view_type_id = data_view_get_type_once ();
		g_once_init_leave (&data_view_type_id__once, data_view_type_id);
	}
	return data_view_type_id__once;
}

static void
_vala_data_view_get_property (GObject * object,
                              guint property_id,
                              GValue * value,
                              GParamSpec * pspec)
{
	DataView * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_DATA_VIEW, DataView);
	switch (property_id) {
		case DATA_VIEW_SOURCE_PROPERTY:
		g_value_set_object (value, data_view_get_source (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_data_view_set_property (GObject * object,
                              guint property_id,
                              const GValue * value,
                              GParamSpec * pspec)
{
	DataView * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, TYPE_DATA_VIEW, DataView);
	switch (property_id) {
		case DATA_VIEW_SOURCE_PROPERTY:
		data_view_set_source (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

