/* pantheon-photos-transitions.h generated by valac 0.56.19, the Vala compiler, do not modify */

#ifndef __PLUGINS_PANTHEON_PHOTOS_TRANSITIONS_PANTHEON_PHOTOS_TRANSITIONS_H__
#define __PLUGINS_PANTHEON_PHOTOS_TRANSITIONS_PANTHEON_PHOTOS_TRANSITIONS_H__

#include "io.elementary.photos.h"
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <gio/gio.h>
#include <glib/gi18n-lib.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif
#define RESOURCES_WEBSITE_NAME _ ("Visit the Yorba web site")
#define RESOURCES_WEBSITE_URL "http://www.yorba.org"
#define RESOURCES_LICENSE "\n" \
"                              Photos is free software; you can redistr" \
"ibute it and/or modify it under the\n" \
"                              terms of the GNU Lesser General Public L" \
"icense as published by the Free\n" \
"                              Software Foundation; either version 2.1 " \
"of the License, or (at your option)\n" \
"                              any later version.\n" \
"\n" \
"                              Photos is distributed in the hope that i" \
"t will be useful, but WITHOUT\n" \
"                              ANY WARRANTY; without even the implied w" \
"arranty of MERCHANTABILITY or FITNESS\n" \
"                              FOR A PARTICULAR PURPOSE.  See the GNU L" \
"esser General Public License for\n" \
"                              more details.\n" \
"\n" \
"                              You should have received a copy of the G" \
"NU Lesser General Public License\n" \
"                              along with Photos; if not, write to the " \
"Free Software Foundation, Inc.,\n" \
"                              51 Franklin St, Fifth Floor, Boston, MA " \
" 02110-1301 USA\n" \
"                              "
#define RESOURCES_TRANSLATORS _ ("translator-credits")

#define TYPE_SHOTWELL_TRANSITION_DESCRIPTOR (shotwell_transition_descriptor_get_type ())
#define SHOTWELL_TRANSITION_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SHOTWELL_TRANSITION_DESCRIPTOR, ShotwellTransitionDescriptor))
#define SHOTWELL_TRANSITION_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SHOTWELL_TRANSITION_DESCRIPTOR, ShotwellTransitionDescriptorClass))
#define IS_SHOTWELL_TRANSITION_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SHOTWELL_TRANSITION_DESCRIPTOR))
#define IS_SHOTWELL_TRANSITION_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SHOTWELL_TRANSITION_DESCRIPTOR))
#define SHOTWELL_TRANSITION_DESCRIPTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SHOTWELL_TRANSITION_DESCRIPTOR, ShotwellTransitionDescriptorClass))

typedef struct _ShotwellTransitionDescriptor ShotwellTransitionDescriptor;
typedef struct _ShotwellTransitionDescriptorClass ShotwellTransitionDescriptorClass;
typedef struct _ShotwellTransitionDescriptorPrivate ShotwellTransitionDescriptorPrivate;

struct _ShotwellTransitionDescriptor {
	GObject parent_instance;
	ShotwellTransitionDescriptorPrivate * priv;
};

struct _ShotwellTransitionDescriptorClass {
	GObjectClass parent_class;
	const gchar* (*get_id) (ShotwellTransitionDescriptor* self);
	const gchar* (*get_pluggable_name) (ShotwellTransitionDescriptor* self);
	SpitTransitionsEffect* (*create) (ShotwellTransitionDescriptor* self, SpitHostInterface* host);
};

VALA_EXTERN SpitModule* spit_entry_point (SpitEntryPointParams* params);
VALA_EXTERN GType shotwell_transition_descriptor_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ShotwellTransitionDescriptor, g_object_unref)
VALA_EXTERN ShotwellTransitionDescriptor* shotwell_transition_descriptor_construct (GType object_type,
                                                                        GFile* resource_directory);
VALA_EXTERN const gchar* shotwell_transition_descriptor_get_id (ShotwellTransitionDescriptor* self);
VALA_EXTERN const gchar* shotwell_transition_descriptor_get_pluggable_name (ShotwellTransitionDescriptor* self);
VALA_EXTERN SpitTransitionsEffect* shotwell_transition_descriptor_create (ShotwellTransitionDescriptor* self,
                                                              SpitHostInterface* host);
VALA_EXTERN GdkPixbuf** resources_load_icon_set (GFile* icon_file,
                                     gint* result_length1);

G_END_DECLS

#endif
