/* Config.c generated by valac 0.56.19, the Vala compiler
 * generated from Config.vala, do not modify */

/*
* Copyright (c) 2011-2013 Yorba Foundation
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*/

#include "io.elementary.photos.h"
#include <glib.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	CONFIG_FACADE_0_PROPERTY,
	CONFIG_FACADE_NUM_PROPERTIES
};
static GParamSpec* config_facade_properties[CONFIG_FACADE_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

static gpointer config_facade_parent_class = NULL;
static ConfigFacade* config_facade_instance;
static ConfigFacade* config_facade_instance = NULL;

static ConfigFacade* config_facade_new (void);
static ConfigFacade* config_facade_construct (GType object_type);
static void config_facade_finalize (GObject * obj);
static GType config_facade_get_type_once (void);

static ConfigFacade*
config_facade_construct (GType object_type)
{
	ConfigFacade * self = NULL;
	GSettingsConfigurationEngine* _tmp0_;
	GSettingsConfigurationEngine* _tmp1_;
	_tmp0_ = gsettings_configuration_engine_new ();
	_tmp1_ = _tmp0_;
	self = (ConfigFacade*) configuration_facade_construct (object_type, (ConfigurationEngine*) _tmp1_);
	_g_object_unref0 (_tmp1_);
	return self;
}

static ConfigFacade*
config_facade_new (void)
{
	return config_facade_construct (CONFIG_TYPE_FACADE);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

ConfigFacade*
config_facade_get_instance (void)
{
	ConfigFacade* _tmp0_;
	ConfigFacade* _tmp2_;
	ConfigFacade* _tmp3_;
	ConfigFacade* result;
	_tmp0_ = config_facade_instance;
	if (_tmp0_ == NULL) {
		ConfigFacade* _tmp1_;
		_tmp1_ = config_facade_new ();
		_g_object_unref0 (config_facade_instance);
		config_facade_instance = _tmp1_;
	}
	_tmp2_ = config_facade_instance;
	_tmp3_ = _g_object_ref0 (_tmp2_);
	result = _tmp3_;
	return result;
}

static void
config_facade_class_init (ConfigFacadeClass * klass,
                          gpointer klass_data)
{
	config_facade_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = config_facade_finalize;
}

static void
config_facade_instance_init (ConfigFacade * self,
                             gpointer klass)
{
}

static void
config_facade_finalize (GObject * obj)
{
	ConfigFacade * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CONFIG_TYPE_FACADE, ConfigFacade);
	G_OBJECT_CLASS (config_facade_parent_class)->finalize (obj);
}

static GType
config_facade_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ConfigFacadeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) config_facade_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ConfigFacade), 0, (GInstanceInitFunc) config_facade_instance_init, NULL };
	GType config_facade_type_id;
	config_facade_type_id = g_type_register_static (TYPE_CONFIGURATION_FACADE, "ConfigFacade", &g_define_type_info, 0);
	return config_facade_type_id;
}

GType
config_facade_get_type (void)
{
	static volatile gsize config_facade_type_id__once = 0;
	if (g_once_init_enter (&config_facade_type_id__once)) {
		GType config_facade_type_id;
		config_facade_type_id = config_facade_get_type_once ();
		g_once_init_leave (&config_facade_type_id__once, config_facade_type_id);
	}
	return config_facade_type_id__once;
}

