/* ConfigurationInterfaces.c generated by valac 0.56.19, the Vala compiler
 * generated from ConfigurationInterfaces.vala, do not modify */

/*
* Copyright (c) 2009-2013 Yorba Foundation
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*/

#include "io.elementary.photos.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <float.h>
#include <math.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	CONFIGURATION_FACADE_0_PROPERTY,
	CONFIGURATION_FACADE_NUM_PROPERTIES
};
static GParamSpec* configuration_facade_properties[CONFIGURATION_FACADE_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

struct _ConfigurationFacadePrivate {
	ConfigurationEngine* engine;
};

static gint ConfigurationFacade_private_offset;
static gpointer configuration_facade_parent_class = NULL;

static GType configuration_engine_get_type_once (void);
static gboolean configuration_facade_real_get_plugin_bool (ConfigurationFacade* self,
                                                    const gchar* domain,
                                                    const gchar* id,
                                                    const gchar* key,
                                                    gboolean def);
static void configuration_facade_real_set_plugin_bool (ConfigurationFacade* self,
                                                const gchar* domain,
                                                const gchar* id,
                                                const gchar* key,
                                                gboolean val);
static gdouble configuration_facade_real_get_plugin_double (ConfigurationFacade* self,
                                                     const gchar* domain,
                                                     const gchar* id,
                                                     const gchar* key,
                                                     gdouble def);
static void configuration_facade_real_set_plugin_double (ConfigurationFacade* self,
                                                  const gchar* domain,
                                                  const gchar* id,
                                                  const gchar* key,
                                                  gdouble val);
static gint configuration_facade_real_get_plugin_int (ConfigurationFacade* self,
                                               const gchar* domain,
                                               const gchar* id,
                                               const gchar* key,
                                               gint def);
static void configuration_facade_real_set_plugin_int (ConfigurationFacade* self,
                                               const gchar* domain,
                                               const gchar* id,
                                               const gchar* key,
                                               gint val);
static gchar* configuration_facade_real_get_plugin_string (ConfigurationFacade* self,
                                                    const gchar* domain,
                                                    const gchar* id,
                                                    const gchar* key,
                                                    const gchar* def);
static void configuration_facade_real_set_plugin_string (ConfigurationFacade* self,
                                                  const gchar* domain,
                                                  const gchar* id,
                                                  const gchar* key,
                                                  const gchar* val);
static void configuration_facade_real_unset_plugin_key (ConfigurationFacade* self,
                                                 const gchar* domain,
                                                 const gchar* id,
                                                 const gchar* key);
static void configuration_facade_finalize (GObject * obj);
static GType configuration_facade_get_type_once (void);

GQuark
configuration_error_quark (void)
{
	return g_quark_from_static_string ("configuration-error-quark");
}

static GType
configuration_error_get_type_once (void)
{
	static const GEnumValue values[] = {{CONFIGURATION_ERROR_PROPERTY_HAS_NO_VALUE, "CONFIGURATION_ERROR_PROPERTY_HAS_NO_VALUE", "property-has-no-value"}, {CONFIGURATION_ERROR_ENGINE_ERROR, "CONFIGURATION_ERROR_ENGINE_ERROR", "engine-error"}, {0, NULL, NULL}};
	GType configuration_error_type_id;
	configuration_error_type_id = g_enum_register_static ("ConfigurationError", values);
	return configuration_error_type_id;
}

GType
configuration_error_get_type (void)
{
	static volatile gsize configuration_error_type_id__once = 0;
	if (g_once_init_enter (&configuration_error_type_id__once)) {
		GType configuration_error_type_id;
		configuration_error_type_id = configuration_error_get_type_once ();
		g_once_init_leave (&configuration_error_type_id__once, configuration_error_type_id);
	}
	return configuration_error_type_id__once;
}

gboolean
configuration_engine_get_plugin_bool (ConfigurationEngine* self,
                                      const gchar* domain,
                                      const gchar* id,
                                      const gchar* key,
                                      gboolean def)
{
	ConfigurationEngineIface* _iface_;
	g_return_val_if_fail (self != NULL, FALSE);
	_iface_ = CONFIGURATION_ENGINE_GET_INTERFACE (self);
	if (_iface_->get_plugin_bool) {
		return _iface_->get_plugin_bool (self, domain, id, key, def);
	}
	return FALSE;
}

void
configuration_engine_set_plugin_bool (ConfigurationEngine* self,
                                      const gchar* domain,
                                      const gchar* id,
                                      const gchar* key,
                                      gboolean val)
{
	ConfigurationEngineIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = CONFIGURATION_ENGINE_GET_INTERFACE (self);
	if (_iface_->set_plugin_bool) {
		_iface_->set_plugin_bool (self, domain, id, key, val);
	}
}

gdouble
configuration_engine_get_plugin_double (ConfigurationEngine* self,
                                        const gchar* domain,
                                        const gchar* id,
                                        const gchar* key,
                                        gdouble def)
{
	ConfigurationEngineIface* _iface_;
	g_return_val_if_fail (self != NULL, -1.0);
	_iface_ = CONFIGURATION_ENGINE_GET_INTERFACE (self);
	if (_iface_->get_plugin_double) {
		return _iface_->get_plugin_double (self, domain, id, key, def);
	}
	return -1.0;
}

void
configuration_engine_set_plugin_double (ConfigurationEngine* self,
                                        const gchar* domain,
                                        const gchar* id,
                                        const gchar* key,
                                        gdouble val)
{
	ConfigurationEngineIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = CONFIGURATION_ENGINE_GET_INTERFACE (self);
	if (_iface_->set_plugin_double) {
		_iface_->set_plugin_double (self, domain, id, key, val);
	}
}

gint
configuration_engine_get_plugin_int (ConfigurationEngine* self,
                                     const gchar* domain,
                                     const gchar* id,
                                     const gchar* key,
                                     gint def)
{
	ConfigurationEngineIface* _iface_;
	g_return_val_if_fail (self != NULL, -1);
	_iface_ = CONFIGURATION_ENGINE_GET_INTERFACE (self);
	if (_iface_->get_plugin_int) {
		return _iface_->get_plugin_int (self, domain, id, key, def);
	}
	return -1;
}

void
configuration_engine_set_plugin_int (ConfigurationEngine* self,
                                     const gchar* domain,
                                     const gchar* id,
                                     const gchar* key,
                                     gint val)
{
	ConfigurationEngineIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = CONFIGURATION_ENGINE_GET_INTERFACE (self);
	if (_iface_->set_plugin_int) {
		_iface_->set_plugin_int (self, domain, id, key, val);
	}
}

gchar*
configuration_engine_get_plugin_string (ConfigurationEngine* self,
                                        const gchar* domain,
                                        const gchar* id,
                                        const gchar* key,
                                        const gchar* def)
{
	ConfigurationEngineIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = CONFIGURATION_ENGINE_GET_INTERFACE (self);
	if (_iface_->get_plugin_string) {
		return _iface_->get_plugin_string (self, domain, id, key, def);
	}
	return NULL;
}

void
configuration_engine_set_plugin_string (ConfigurationEngine* self,
                                        const gchar* domain,
                                        const gchar* id,
                                        const gchar* key,
                                        const gchar* val)
{
	ConfigurationEngineIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = CONFIGURATION_ENGINE_GET_INTERFACE (self);
	if (_iface_->set_plugin_string) {
		_iface_->set_plugin_string (self, domain, id, key, val);
	}
}

void
configuration_engine_unset_plugin_key (ConfigurationEngine* self,
                                       const gchar* domain,
                                       const gchar* id,
                                       const gchar* key)
{
	ConfigurationEngineIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = CONFIGURATION_ENGINE_GET_INTERFACE (self);
	if (_iface_->unset_plugin_key) {
		_iface_->unset_plugin_key (self, domain, id, key);
	}
}

static void
configuration_engine_default_init (ConfigurationEngineIface * iface,
                                   gpointer iface_data)
{
}

static GType
configuration_engine_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ConfigurationEngineIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) configuration_engine_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType configuration_engine_type_id;
	configuration_engine_type_id = g_type_register_static (G_TYPE_INTERFACE, "ConfigurationEngine", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (configuration_engine_type_id, G_TYPE_OBJECT);
	return configuration_engine_type_id;
}

GType
configuration_engine_get_type (void)
{
	static volatile gsize configuration_engine_type_id__once = 0;
	if (g_once_init_enter (&configuration_engine_type_id__once)) {
		GType configuration_engine_type_id;
		configuration_engine_type_id = configuration_engine_get_type_once ();
		g_once_init_leave (&configuration_engine_type_id__once, configuration_engine_type_id);
	}
	return configuration_engine_type_id__once;
}

static inline gpointer
configuration_facade_get_instance_private (ConfigurationFacade* self)
{
	return G_STRUCT_MEMBER_P (self, ConfigurationFacade_private_offset);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

ConfigurationFacade*
configuration_facade_construct (GType object_type,
                                ConfigurationEngine* engine)
{
	ConfigurationFacade * self = NULL;
	ConfigurationEngine* _tmp0_;
	g_return_val_if_fail (engine != NULL, NULL);
	self = (ConfigurationFacade*) g_object_new (object_type, NULL);
	_tmp0_ = _g_object_ref0 (engine);
	_g_object_unref0 (self->priv->engine);
	self->priv->engine = _tmp0_;
	return self;
}

ConfigurationEngine*
configuration_facade_get_engine (ConfigurationFacade* self)
{
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	ConfigurationEngine* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->engine;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	result = _tmp1_;
	return result;
}

static gboolean
configuration_facade_real_get_plugin_bool (ConfigurationFacade* self,
                                           const gchar* domain,
                                           const gchar* id,
                                           const gchar* key,
                                           gboolean def)
{
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	gboolean _tmp2_;
	gboolean result;
	g_return_val_if_fail (domain != NULL, FALSE);
	g_return_val_if_fail (id != NULL, FALSE);
	g_return_val_if_fail (key != NULL, FALSE);
	_tmp0_ = configuration_facade_get_engine (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = configuration_engine_get_plugin_bool (_tmp1_, domain, id, key, def);
	_g_object_unref0 (_tmp1_);
	result = _tmp2_;
	return result;
}

gboolean
configuration_facade_get_plugin_bool (ConfigurationFacade* self,
                                      const gchar* domain,
                                      const gchar* id,
                                      const gchar* key,
                                      gboolean def)
{
	ConfigurationFacadeClass* _klass_;
	g_return_val_if_fail (self != NULL, FALSE);
	_klass_ = CONFIGURATION_FACADE_GET_CLASS (self);
	if (_klass_->get_plugin_bool) {
		return _klass_->get_plugin_bool (self, domain, id, key, def);
	}
	return FALSE;
}

static void
configuration_facade_real_set_plugin_bool (ConfigurationFacade* self,
                                           const gchar* domain,
                                           const gchar* id,
                                           const gchar* key,
                                           gboolean val)
{
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	g_return_if_fail (domain != NULL);
	g_return_if_fail (id != NULL);
	g_return_if_fail (key != NULL);
	_tmp0_ = configuration_facade_get_engine (self);
	_tmp1_ = _tmp0_;
	configuration_engine_set_plugin_bool (_tmp1_, domain, id, key, val);
	_g_object_unref0 (_tmp1_);
}

void
configuration_facade_set_plugin_bool (ConfigurationFacade* self,
                                      const gchar* domain,
                                      const gchar* id,
                                      const gchar* key,
                                      gboolean val)
{
	ConfigurationFacadeClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = CONFIGURATION_FACADE_GET_CLASS (self);
	if (_klass_->set_plugin_bool) {
		_klass_->set_plugin_bool (self, domain, id, key, val);
	}
}

static gdouble
configuration_facade_real_get_plugin_double (ConfigurationFacade* self,
                                             const gchar* domain,
                                             const gchar* id,
                                             const gchar* key,
                                             gdouble def)
{
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	gdouble _tmp2_;
	gdouble result;
	g_return_val_if_fail (domain != NULL, -1.0);
	g_return_val_if_fail (id != NULL, -1.0);
	g_return_val_if_fail (key != NULL, -1.0);
	_tmp0_ = configuration_facade_get_engine (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = configuration_engine_get_plugin_double (_tmp1_, domain, id, key, def);
	_g_object_unref0 (_tmp1_);
	result = _tmp2_;
	return result;
}

gdouble
configuration_facade_get_plugin_double (ConfigurationFacade* self,
                                        const gchar* domain,
                                        const gchar* id,
                                        const gchar* key,
                                        gdouble def)
{
	ConfigurationFacadeClass* _klass_;
	g_return_val_if_fail (self != NULL, -1.0);
	_klass_ = CONFIGURATION_FACADE_GET_CLASS (self);
	if (_klass_->get_plugin_double) {
		return _klass_->get_plugin_double (self, domain, id, key, def);
	}
	return -1.0;
}

static void
configuration_facade_real_set_plugin_double (ConfigurationFacade* self,
                                             const gchar* domain,
                                             const gchar* id,
                                             const gchar* key,
                                             gdouble val)
{
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	g_return_if_fail (domain != NULL);
	g_return_if_fail (id != NULL);
	g_return_if_fail (key != NULL);
	_tmp0_ = configuration_facade_get_engine (self);
	_tmp1_ = _tmp0_;
	configuration_engine_set_plugin_double (_tmp1_, domain, id, key, val);
	_g_object_unref0 (_tmp1_);
}

void
configuration_facade_set_plugin_double (ConfigurationFacade* self,
                                        const gchar* domain,
                                        const gchar* id,
                                        const gchar* key,
                                        gdouble val)
{
	ConfigurationFacadeClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = CONFIGURATION_FACADE_GET_CLASS (self);
	if (_klass_->set_plugin_double) {
		_klass_->set_plugin_double (self, domain, id, key, val);
	}
}

static gint
configuration_facade_real_get_plugin_int (ConfigurationFacade* self,
                                          const gchar* domain,
                                          const gchar* id,
                                          const gchar* key,
                                          gint def)
{
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	gint _tmp2_;
	gint result;
	g_return_val_if_fail (domain != NULL, -1);
	g_return_val_if_fail (id != NULL, -1);
	g_return_val_if_fail (key != NULL, -1);
	_tmp0_ = configuration_facade_get_engine (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = configuration_engine_get_plugin_int (_tmp1_, domain, id, key, def);
	_g_object_unref0 (_tmp1_);
	result = _tmp2_;
	return result;
}

gint
configuration_facade_get_plugin_int (ConfigurationFacade* self,
                                     const gchar* domain,
                                     const gchar* id,
                                     const gchar* key,
                                     gint def)
{
	ConfigurationFacadeClass* _klass_;
	g_return_val_if_fail (self != NULL, -1);
	_klass_ = CONFIGURATION_FACADE_GET_CLASS (self);
	if (_klass_->get_plugin_int) {
		return _klass_->get_plugin_int (self, domain, id, key, def);
	}
	return -1;
}

static void
configuration_facade_real_set_plugin_int (ConfigurationFacade* self,
                                          const gchar* domain,
                                          const gchar* id,
                                          const gchar* key,
                                          gint val)
{
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	g_return_if_fail (domain != NULL);
	g_return_if_fail (id != NULL);
	g_return_if_fail (key != NULL);
	_tmp0_ = configuration_facade_get_engine (self);
	_tmp1_ = _tmp0_;
	configuration_engine_set_plugin_int (_tmp1_, domain, id, key, val);
	_g_object_unref0 (_tmp1_);
}

void
configuration_facade_set_plugin_int (ConfigurationFacade* self,
                                     const gchar* domain,
                                     const gchar* id,
                                     const gchar* key,
                                     gint val)
{
	ConfigurationFacadeClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = CONFIGURATION_FACADE_GET_CLASS (self);
	if (_klass_->set_plugin_int) {
		_klass_->set_plugin_int (self, domain, id, key, val);
	}
}

static gchar*
configuration_facade_real_get_plugin_string (ConfigurationFacade* self,
                                             const gchar* domain,
                                             const gchar* id,
                                             const gchar* key,
                                             const gchar* def)
{
	gchar* _result_ = NULL;
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	const gchar* _tmp4_ = NULL;
	const gchar* _tmp5_;
	gchar* _tmp7_;
	gchar* result;
	g_return_val_if_fail (domain != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	g_return_val_if_fail (key != NULL, NULL);
	_tmp0_ = configuration_facade_get_engine (self);
	_tmp1_ = _tmp0_;
	_tmp2_ = configuration_engine_get_plugin_string (_tmp1_, domain, id, key, def);
	_tmp3_ = _tmp2_;
	_g_object_unref0 (_tmp1_);
	_result_ = _tmp3_;
	_tmp5_ = _result_;
	if (g_strcmp0 (_tmp5_, "") == 0) {
		_tmp4_ = NULL;
	} else {
		const gchar* _tmp6_;
		_tmp6_ = _result_;
		_tmp4_ = _tmp6_;
	}
	_tmp7_ = g_strdup (_tmp4_);
	result = _tmp7_;
	_g_free0 (_result_);
	return result;
}

gchar*
configuration_facade_get_plugin_string (ConfigurationFacade* self,
                                        const gchar* domain,
                                        const gchar* id,
                                        const gchar* key,
                                        const gchar* def)
{
	ConfigurationFacadeClass* _klass_;
	g_return_val_if_fail (self != NULL, NULL);
	_klass_ = CONFIGURATION_FACADE_GET_CLASS (self);
	if (_klass_->get_plugin_string) {
		return _klass_->get_plugin_string (self, domain, id, key, def);
	}
	return NULL;
}

static void
configuration_facade_real_set_plugin_string (ConfigurationFacade* self,
                                             const gchar* domain,
                                             const gchar* id,
                                             const gchar* key,
                                             const gchar* val)
{
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	g_return_if_fail (domain != NULL);
	g_return_if_fail (id != NULL);
	g_return_if_fail (key != NULL);
	if (val == NULL) {
		val = "";
	}
	_tmp0_ = configuration_facade_get_engine (self);
	_tmp1_ = _tmp0_;
	configuration_engine_set_plugin_string (_tmp1_, domain, id, key, val);
	_g_object_unref0 (_tmp1_);
}

void
configuration_facade_set_plugin_string (ConfigurationFacade* self,
                                        const gchar* domain,
                                        const gchar* id,
                                        const gchar* key,
                                        const gchar* val)
{
	ConfigurationFacadeClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = CONFIGURATION_FACADE_GET_CLASS (self);
	if (_klass_->set_plugin_string) {
		_klass_->set_plugin_string (self, domain, id, key, val);
	}
}

static void
configuration_facade_real_unset_plugin_key (ConfigurationFacade* self,
                                            const gchar* domain,
                                            const gchar* id,
                                            const gchar* key)
{
	ConfigurationEngine* _tmp0_;
	ConfigurationEngine* _tmp1_;
	g_return_if_fail (domain != NULL);
	g_return_if_fail (id != NULL);
	g_return_if_fail (key != NULL);
	_tmp0_ = configuration_facade_get_engine (self);
	_tmp1_ = _tmp0_;
	configuration_engine_unset_plugin_key (_tmp1_, domain, id, key);
	_g_object_unref0 (_tmp1_);
}

void
configuration_facade_unset_plugin_key (ConfigurationFacade* self,
                                       const gchar* domain,
                                       const gchar* id,
                                       const gchar* key)
{
	ConfigurationFacadeClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = CONFIGURATION_FACADE_GET_CLASS (self);
	if (_klass_->unset_plugin_key) {
		_klass_->unset_plugin_key (self, domain, id, key);
	}
}

static void
configuration_facade_class_init (ConfigurationFacadeClass * klass,
                                 gpointer klass_data)
{
	configuration_facade_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &ConfigurationFacade_private_offset);
	((ConfigurationFacadeClass *) klass)->get_plugin_bool = (gboolean (*) (ConfigurationFacade*, const gchar*, const gchar*, const gchar*, gboolean)) configuration_facade_real_get_plugin_bool;
	((ConfigurationFacadeClass *) klass)->set_plugin_bool = (void (*) (ConfigurationFacade*, const gchar*, const gchar*, const gchar*, gboolean)) configuration_facade_real_set_plugin_bool;
	((ConfigurationFacadeClass *) klass)->get_plugin_double = (gdouble (*) (ConfigurationFacade*, const gchar*, const gchar*, const gchar*, gdouble)) configuration_facade_real_get_plugin_double;
	((ConfigurationFacadeClass *) klass)->set_plugin_double = (void (*) (ConfigurationFacade*, const gchar*, const gchar*, const gchar*, gdouble)) configuration_facade_real_set_plugin_double;
	((ConfigurationFacadeClass *) klass)->get_plugin_int = (gint (*) (ConfigurationFacade*, const gchar*, const gchar*, const gchar*, gint)) configuration_facade_real_get_plugin_int;
	((ConfigurationFacadeClass *) klass)->set_plugin_int = (void (*) (ConfigurationFacade*, const gchar*, const gchar*, const gchar*, gint)) configuration_facade_real_set_plugin_int;
	((ConfigurationFacadeClass *) klass)->get_plugin_string = (gchar* (*) (ConfigurationFacade*, const gchar*, const gchar*, const gchar*, const gchar*)) configuration_facade_real_get_plugin_string;
	((ConfigurationFacadeClass *) klass)->set_plugin_string = (void (*) (ConfigurationFacade*, const gchar*, const gchar*, const gchar*, const gchar*)) configuration_facade_real_set_plugin_string;
	((ConfigurationFacadeClass *) klass)->unset_plugin_key = (void (*) (ConfigurationFacade*, const gchar*, const gchar*, const gchar*)) configuration_facade_real_unset_plugin_key;
	G_OBJECT_CLASS (klass)->finalize = configuration_facade_finalize;
}

static void
configuration_facade_instance_init (ConfigurationFacade * self,
                                    gpointer klass)
{
	self->priv = configuration_facade_get_instance_private (self);
}

static void
configuration_facade_finalize (GObject * obj)
{
	ConfigurationFacade * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_CONFIGURATION_FACADE, ConfigurationFacade);
	_g_object_unref0 (self->priv->engine);
	G_OBJECT_CLASS (configuration_facade_parent_class)->finalize (obj);
}

static GType
configuration_facade_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ConfigurationFacadeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) configuration_facade_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ConfigurationFacade), 0, (GInstanceInitFunc) configuration_facade_instance_init, NULL };
	GType configuration_facade_type_id;
	configuration_facade_type_id = g_type_register_static (G_TYPE_OBJECT, "ConfigurationFacade", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	ConfigurationFacade_private_offset = g_type_add_instance_private (configuration_facade_type_id, sizeof (ConfigurationFacadePrivate));
	return configuration_facade_type_id;
}

GType
configuration_facade_get_type (void)
{
	static volatile gsize configuration_facade_type_id__once = 0;
	if (g_once_init_enter (&configuration_facade_type_id__once)) {
		GType configuration_facade_type_id;
		configuration_facade_type_id = configuration_facade_get_type_once ();
		g_once_init_leave (&configuration_facade_type_id__once, configuration_facade_type_id);
	}
	return configuration_facade_type_id__once;
}

